require 'net/http'

module AlacrityRails
  class Client
    def self.open_transaction(transaction)
      transactions.push(transaction) if config_enabled?
    end

    def self.store_metadata(data)
      transactions.each do |transaction|
        transaction.store_metadata(data)
      end
    end

    def self.store_timeline_event(data)
      transactions.each do |transaction|
        transaction.store_timeline_event(data)
      end
    end

    def self.close_transaction!(response)
      if transaction = transactions.pop
        transaction.finalize(response)
        transmit(transaction)
      end
    end

    def self.log_server_startup
      transmit(AlacrityRails::Transaction::ServerStartup.new)
    end

    def self.transactions
      Thread.current[:alacrity_transactions] ||= []
    end

    def self.reset
      Thread.current[:alacrity_transactions] = []
    end

    def self.transmit(transactable)
      Thread.new do
        begin
          Net::HTTP.start(*transactable.net_http_start_arguments) do |http|
            http.request(transactable.post_request)
          end
        rescue
        end
      end
    end

    def self.config_enabled?
      @config_enabled = Config.enabled? if @config_enabled == nil
      @config_enabled
    end

  end
end
