module AlacrityRails
  module Probe
    class ActiveJob
      def self.activate
        return unless defined?(::ActiveJob)

        ActiveSupport::Notifications.subscribe 'enqueue.active_job' do |name, started, finished, unique_id, data|
          AlacrityRails::Client.store_timeline_event(
            name: 'Job Queued',
            event_type: 'job',
            engine: 'ActiveJob',
            started_at: started,
            finished_at: finished,
            detail: data[:job].class.to_s
          )
        end

        ::ActiveJob::Base.include(ActiveJobMixin)
      end
    end

    module ActiveJobMixin
      extend ActiveSupport::Concern

      included do
        around_perform :alacrity_instrumentor

        private

        def alacrity_instrumentor
          AlacrityRails::Instrumentor.instrument(
            type: AlacrityRails::Transaction::Job,
            data: {
              name: self.class.to_s,
              args: @arguments
            }
          ) do
            yield
          end
        end
      end
    end
  end
end
