module AlacrityRails
  module Probe
    class MongoDriver
      def self.activate
        if defined?(Mongo)
          Mongo::Monitoring::Global.subscribe(Mongo::Monitoring::COMMAND, new)
        end
      end

      def start_events
        @start_events ||= {}
      end

      def started(event)
        start_events[event.operation_id] = { started_at: DateTime.now, command: event.command }
      end

      def succeeded(event)
        finished = DateTime.now
        start_event = start_events.delete(event.operation_id)

        command = {}
        start_event[:command].each do |key, value|
          command[key] = value.respond_to?(:keys) ? value.keys : value
        end

        store(event.command_name, start_event[:started_at], finished, command)
      rescue => error
        Logger.error("Alacrity mongo driver tracking failed: #{error.inspect}")
      end
      alias failed succeeded

      protected

      def store(command_name, started_at, finished_at, command)
        AlacrityRails::Client.store_timeline_event(
          name: command_name,
          event_type: 'database',
          engine: 'Mongo',
          started_at: started_at,
          finished_at: finished_at,
          detail: command.to_s
        )
      end
    end
  end
end
