module AlacrityRails
  module Probe
    class SidekiqServerMiddleware
      ACTIVE_JOB_WRAPPER = /^ActiveJob::QueueAdapters::.+$/

      def call(worker_instance, msg, queue)
        if ACTIVE_JOB_WRAPPER.match(msg['class'])
          yield
        else
          AlacrityRails::Instrumentor.instrument(
            type: AlacrityRails::Transaction::Job,
            data: {
              name: msg['class'],
              args: msg['args'],
              error_class: msg['error_class'],
              error_message: msg['error_message']
            }
          ) do
            yield
          end
        end
      end
    end

    class Sidekiq
      def self.activate
        if defined?(::Sidekiq) && AlacrityRails::Config.sidekiq_enabled?
          ::Sidekiq.configure_server do |config|
            config.server_middleware do |chain|
              chain.add SidekiqServerMiddleware
            end
          end
        end
      end
    end
  end
end
