module AlacrityRails::Transaction
  class Base
    def initialize(data={})
       # May be overridden
    end

    def finalize(response)
      # May be overridden
    end

    def post_request
      Net::HTTP::Post.new(endpoint, {
        "Authorization" => authorization_header_value,
        "Content-Type" => "application/json"
      }).tap { |request| request.body = to_json }
    end

    def self.net_http_start_arguments
      @net_http_start_arguments ||= [endpoint.host, endpoint.port, use_ssl: endpoint.scheme == 'https']
    end
    def net_http_start_arguments; self.class.net_http_start_arguments end

    def self.endpoint
      raise 'Must implement'
    end
    def endpoint; self.class.endpoint end

    def self.authorization_header_value
      @authorization_header_value ||= ActionController::HttpAuthentication::Token.encode_credentials(AlacrityRails::Config.api_token)
    end
    def authorization_header_value; self.class.authorization_header_value end

    def store_metadata(data)
      # May be overridden
    end

    def store_timeline_event(data)
      timeline_events << {
        name: data[:name],
        event_type: data[:event_type],
        engine: data[:engine],
        started_at: data[:started_at],
        finished_at: data[:finished_at],
        detail: data[:detail]
      }
    end

    def timeline_events
      @timeline_events ||= []
    end

    def prepared_timeline_events
      timeline_events.uniq do |timeline_event|
        timeline_event.values_at(:engine, :started_at, :finished_at, :detail).join('~')
      end.map do |timeline_event|
        timeline_event[:started_at] = absolute_time(timeline_event[:started_at].to_datetime)
        timeline_event[:finished_at] = absolute_time(timeline_event[:finished_at].to_datetime)
        timeline_event
      end
    end

    def absolute_time(datetime)
      datetime.strftime('%Q').to_i
    end
  end
end
