module AlacrityRails::Transaction
  class WebRequest < Base
    attr_accessor :url, :name, :format, :method, :status

    def initialize(data={})
      @middleware_started_at = DateTime.now
      self.url = data[:url]
      self.method = data[:method]
    end

    def store_metadata(data)
      self.name = [data[:controller], data[:action]].reject(&:blank?).join('#')
      self.format = data[:format]
    end

    def finalize(response)
      self.status = response&.first
      @middleware_finished_at = DateTime.now
    end

    def as_json(*args)
      middleware_started_at = absolute_time(@middleware_started_at)
      middleware_finished_at = absolute_time(@middleware_finished_at) || absolute_time(DateTime.now)
      {
        environment: AlacrityRails::ServerConfig.environment,
        event_type: 'web-request',
        name: name,
        started_at: middleware_started_at,
        finished_at: middleware_finished_at,
        timeline_events: prepared_timeline_events,
        data: {
          url: url,
          format: format,
          method: method,
          status: status,
        }
      }
    end

    def self.endpoint
      @endpoint ||= URI("#{AlacrityRails::Config.collector_host}/v3/events")
    end
  end
end
